<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2006 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Plugin.php 4453 2006-03-13 18:14:41Z chris@m3.net $
*/

require_once MAX_PATH . '/lib/Max.php';
require_once MAX_PATH . '/lib/max/FileScanner.php';

/**
 * The default directory to store plugin configution and cache files.
 */
define('MAX_PLUGINS_VAR', MAX_PATH . '/var/plugins');

/**
 * The default write mode for directories inside MAX_PLUGINS_VAR.
 */
define('MAX_PLUGINS_VAR_WRITE_MODE', 0755);

/**
 * The default file mask for plugins.
 */
define('MAX_PLUGINS_EXTENSION', '.plugin.php');
define('MAX_PLUGINS_EXTENSION_ESC', '\.plugin\.php');
define('MAX_PLUGINS_FILE_MASK', '^.*/([a-zA-Z0-9\-_]*)/([a-zA-Z0-9\-_]*)'.MAX_PLUGINS_EXTENSION_ESC.'$');

/**
 * MAX_Plugin is a static helper class for dealing with plugins. It
 * provides a factory method for including/instantiating plugins, and
 * provides methods for:
 *  - Reading groups of plugins from a module or from a modules/package;
 *  - Calling plugin methods;
 *  - Reading and writing plugin configuration files, if they exist; and
 *  - Reading and writing plugin cache files, if they exist.
 *
 * @static
 * @package    MaxPlugins
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Radek Maciaszek <radek@m3.net>
 */
class MAX_Plugin
{

    /**
     * A factory method, for including and instantiating a plugin, given a
     * module/package (and optional plugin name).
     *
     * @static
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package The plugin package name (i.e. /plugins/module/package
     *                        directory).
     * @param string $name Optional name of the PHP file which contains the plugin,
     *                     otherwise the plugin with the same name as the package
     *                     is assumed.
     * @return mixed The instantiated plugin object, or false on error.
     */
    function &factory($module, $package, $name = null)
    {
        if (!MAX_Plugin::_includePluginFile($module, $package, $name)) {
            return false;
        }
        if ($name === null) {
            $name = $package;
        }
        $className = MAX_Plugin::_getPluginClassName($module, $package, $name);
        $obj = new $className($module, $package, $name);
        $obj->module  = $module;
        $obj->package = $package;
        $obj->name    = $name;
        return $obj;
    }

    /**
     * A private method to include a plugin file, given a module/package
     * (and optional plugin name).
     *
     * @static
     * @access private
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package The plugin package name (i.e. /plugins/module/package
     *                        directory).
     * @param string $name Optional name of the PHP file which contains the plugin,
     *                     otherwise the plugin with the same name as the package
     *                     is assumed.
     * @return boolean True on success, false otherwise.
     */
    function _includePluginFile($module, $package, $name = null)
    {
        if ($name === null) {
            $name = $package;
        }
        $fileName = MAX_PATH . "/plugins/$module/$package/$name".MAX_PLUGINS_EXTENSION;
        if (!file_exists($fileName)) {
            MAX::raiseError("Unable to include the file $fileName.");
            return false;
        } else {
            include_once $fileName;
        }
        $className = MAX_Plugin::_getPluginClassName($module, $package, $name);
        if (!class_exists($className)) {
            MAX::raiseError("Plugin file included but class '$className' does not exist.");
            return false;
        } else {
            return true;
        }
    }

    /**
     * A private method for generating the (expected) class name of a plugin.
     *
     * @static
     * @access private
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package The plugin package name (i.e. /plugins/module/package
     *                        directory).
     * @param string $name Optional name of the PHP file which contains the plugin,
     *                     otherwise the plugin with the same name as the package
     *                     is assumed.
     * @return string The plugin class name.
     */
    function _getPluginClassName($module, $package, $name = null)
    {
        if ($name === null) {
            $name = $package;
        }
        $className = "Plugins_" . ucfirst($module) . '_' . ucfirst($package) . '_' . ucfirst($name);
        return $className;
    }

    /**
     * A method to return an array of plugin objects from a selected plugin module
     * or module/package.
     *
     * @static
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package An optional plugin package name (i.e. /plugins/module/package
     *                        directory). If not given, the search for plugin files will start
     *                        at the module directory level.
     * @param boolean $onlyPluginNameAsIndex If true, the array index for the plugins is
     *                                       "pluginName", otherwise the index is of the
     *                                       format is "packageName:pluginName".
     * @param mixed $recursive If the boolean 'true', returns all plugins in the
     *                         given module (and package, if specified), and all
     *                         subdirectories thereof.
     *                         If an integer, returns all plugins in the given
     *                         module (and package, if specified) and subdirectories
     *                         thereof, down to the depth specified by the parameter.
     * @return array An array of plugin objects, indexed as specified by the
     *               $onlyPluginNameAsIndex parameter.
     */
    function &getPlugins($module, $package = null, $onlyPluginNameAsIndex = true, $recursive = 1)
    {
        $plugins = array();
        $pluginFiles = MAX_Plugin::_getPluginsFiles($module, $package, $recursive);
        foreach ($pluginFiles as $key => $pluginFile) {
            $pluginInfo = explode(':', $key);
            if (count($pluginInfo) > 1) {
                if ($onlyPluginNameAsIndex) {
                    $plugins[$pluginInfo[1]] = MAX_Plugin::factory($module, $pluginInfo[0], $pluginInfo[1]);
                } else {
                    $plugins[$key] = MAX_Plugin::factory($module, $pluginInfo[0], $pluginInfo[1]);
                }
            }
        }
        return $plugins;
    }

    /**
     * A private method to return a list of plugin files in a given plugin module,
     * or a given module/package.
     *
     * @static
     * @access private
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package An optional plugin package name (i.e. /plugins/module/package
     *                        directory). If not given, the search for plugin files will
     *                        start at the module directory level.
     * @param mixed $recursive If the boolean 'true', returns all plugin files in the
     *                         given directory and all subdirectories.
     *                         If an integer, returns all plugin files in the given
     *                         directory and subdirectories down to the depth
     *                         specified by the parameter.
     * @return array An array of the plugin files found, indexed by "directory:filename".
     */
    function _getPluginsFiles($module, $package = null, $recursive = 1)
    {
        $pluginsDir = MAX_PATH . '/plugins';
        if (!empty($package)) {
            $dir = $pluginsDir . '/' . $module . '/' . $package;
        } else {
            $dir = $pluginsDir . '/' . $module;
        }
        return MAX_Plugin::_getPluginsFilesFromDirectory($dir, $recursive);
    }

    /**
     * A private method to return a list list of files from a directory
     * (and subdirectories, if appropriate)  which match the defined
     * plugin file mask (MAX_PLUGINS_FILE_MASK).
     *
     * @static
     * @access private
     * @param string $directory The directory to search for files in.
     * @param mixed $recursive If the boolean 'true', returns all files in the given
     *                         directory and all subdirectories that match the file
     *                         mask.
     *                         If an integer, returns all files in the given
     *                         directory and subdirectories down to the depth
     *                         specified by the parameter that match the file mask.
     * @return array An array of the files found, indexed by "directory:filename".
     */
    function _getPluginsFilesFromDirectory($directory, $recursive = 1)
    {
        $oFileScanner = new MAX_FileScanner();
        $oFileScanner->addFileTypes(array('php','inc'));
        $oFileScanner->setFileMask(MAX_PLUGINS_FILE_MASK);
        $oFileScanner->addDir($directory, $recursive);
        return $oFileScanner->getAllFiles();
    }

    /**
     * A method to include a plugin, and call a method statically on the plugin class.
     *
     * @static
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package The plugin package name (i.e. /plugins/module/package
     *                        directory).
     * @param string $name Optional name of the PHP file which contains the plugin,
     *                     otherwise the plugin with the same name as the package
     *                     is assumed.
     * @param string $staticMethod The name of the method of the plugin to call statically.
     * @return mixed The result of the static method call, or false on failure to include
     *               the plugin.
     */
    function &callStaticMethod($module, $package, $name = null, $staticMethod)
    {
        if ($name === null) {
            $name = $package;
        }
        if (!MAX_Plugin::_includePluginFile($module, $package, $name)) {
            return false;
        }
        $className = MAX_Plugin::_getPluginClassName($module, $package, $name);
        $ret = call_user_func(array($className, $staticMethod));
        return $ret;
    }

    /**
     * A method to run one method on group of plugins.
     *
     * @static
     * @param array $aPlugins An array of plugin objects.
     * @param string $methodName The name of the method to executed for every plugin.
     * @return mixed An array of the results of the method calls, or false on error.
     *
     * @TODO: Add ability to pass additional parameters to $callbackMethod?
     */
    function &callOnPlugins(&$aPlugins, $methodName)
    {
        if (!is_array($aPlugins)) {
            MAX::raiseError('Bad argument', MAX_ERROR_INVALIDARGS);
            return false;
        }
        $ret = array();
        foreach ($aPlugins as $key => $plugin) {
            if (is_callable(array($plugin, $methodName))) {
                $ret[$key] = call_user_func(array($aPlugins[$key], $methodName));
            } else {
                $ret[$key] = false;
            }
        }
        return $ret;
    }

    /**
     * A method to read and parse the configuration file for a plugin.
     *
     * @static
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package An optional plugin package name (i.e. /plugins/module/package
     *                        directory). If not given, the module level configuration file
     *                        will be read.
     * @param string $name An optional plugin name (i.e. /plugins/module/package/plugin.plugin.php).
     *                     If not given, the package level configuration file will be read, or
     *                     the module level configuration file will be read, depending on the
     *                     $package parameter.
     * @param boolean $processSections Process section names in the configuration file, and return
     *                                 as a multidimenstional array.
     *                                 {@see http://uk.php.net/manual/en/function.parse-ini-file.php}.
     * @param boolean $copyDefaultIfNotExists If true, copy the default configuration file for the
     *                                        plugin to the real configuration file location, if the
     *                                        real configuration file does not (yet) exist.
     * @return mixed An array containing the parsed configuration file, or false on error.
     *
     */
    function getConfig($module, $package = null, $name = null, $processSections = false, $copyDefaultIfNotExists = true)
    {
        $configFileName = MAX_Plugin::getConfigFileName($module, $package, $name);
        $conf = MAX_Plugin::getConfigByFileName($configFileName, $processSections, false);
        if ($conf !== false) {
            return $conf;
        } elseif ($copyDefaultIfNotExists) {
            MAX_Plugin::copyDefaultConfig($module, $package, $name);
            $defaultConfigFileName = MAX_Plugin::getConfigFileName($module, $package, $name, true);
            return MAX_Plugin::getConfigByFileName($configFileName, $processSections, false);
        }
        MAX::raiseError("Config file '{$configFileName}' doesn't exists", MAX_ERROR_NOFILE);
        return false;
    }

    /**
     * A method to return the file path of the configuration configuration file of a given plugin.
     *
     * @static
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package An optional plugin package name (i.e. /plugins/module/package
     *                        directory). If not given, generates the module level
     *                        configuration file path.
     * @param string $name An optional plugin name (i.e. /plugins/module/package/plugin.plugin.php).
     *                     If not given, generates the package level configuration file path, or
     *                     the module level configuration file path, depending on the $package
     *                     parameter.
     * @param boolean $defaultConfig Optional flag. When true, returns the path to the default
     *                               configuration file distributed with Max; when false, the
     *                               path to the "real" configuration file for the plugin that
     *                               has been written previously is returned.
     * @param string $host An optional parameter to override the host name via which the Max
     *                     installation is currently being accessed.
     * @return string The path to the configuration file.
     */
    function getConfigFileName($module, $package = null, $name = null, $defaultConfig = false, $host = null)
    {
        if ($defaultConfig) {
            if (is_null($host)) {
                $host = 'default';
            }
            $startPath  = MAX_PATH . '/plugins/';
        } else {
            if (is_null($host)) {
                $host = MAX_Plugin::_getHostName();
            }
            $startPath  = MAX_PLUGINS_VAR . '/config/';
        }
        $configName = $host.'.plugin.conf.ini';
        if ($package === null) {
            $configPath = $module . '/';
        } elseif ($name === null) {
            $configPath = $module . '/' . $package.'/';
        } else {
            $configPath = $module . '/' . $package . '/' . $name . '.';
        }

        return $startPath . $configPath . $configName;
    }

    /**
     * A method to read in and parse a plugin configuration file.
     *
     * @static
     * @param string $configFileName The path to the configuration file.
     * @param boolean $processSections Process section names in the configuration file, and return
     *                                 as a multidimenstional array.
     *                                 {@see http://uk.php.net/manual/en/function.parse-ini-file.php}.
     * @param boolean $raiseErrors If true, raise PEAR errors on failure.
     * @return mixed An array containing the parsed configuration file, or false on error.
     */
    function getConfigByFileName($configFileName, $processSections = false, $raiseErrors = true)
    {
        if (file_exists($configFileName)) {
            $conf = parse_ini_file($configFileName, $processSections);
            if (isset($conf['realConfig'])) {
                if (ereg('.*\/(.*)\.plugin\.conf\.ini', $configFileName, $match = null)) {
                    $configFileName = str_replace($match[1], $conf['realConfig'], $configFileName);
                    return MAX_Plugin::getConfigByFileName($configFileName, $processSections, $raiseErrors);
                } else {
                    return false;
                }
            }
            if (is_array($conf)) {
                return $conf;
            } else {
                return false;
            }
        } else {
            if ($raiseErrors) {
                MAX::raiseError("Config file '{$configFileName}' doesn't exists", MAX_ERROR_NOFILE);
            }
            return false;
        }
    }

    /**
     * A method to copy the default configuration file for a plugin (in /plugins) to the
     * real configuration file for that plugin (in /var).
     *
     * @param string $module The plugin module name (i.e. /plugins/module directory).
     * @param string $package An optional plugin package name (i.e. /plugins/module/package
     *                        directory). If not given, copies the module level
     *                        configuration file path.
     * @param string $name An optional plugin name (i.e. /plugins/module/package/plugin.plugin.php).
     *                     If not given, copies the package level configuration file path, or
     *                     the module level configuration file path, depending on the $package
     *                     parameter.
     * @return boolean True on success, false otherwise.
     *
     */
    function copyDefaultConfig($module, $package = null, $name = null)
    {
        $configFileName = MAX_Plugin::getConfigFileName($module, $package, $name);
        $defaultConfigFileName = MAX_Plugin::getConfigFileName($module, $package, $name, $default = true);
        if (file_exists($defaultConfigFileName)) {
            // Create directory (if not exists)
            MAX_Plugin::_mkDirRecursive(dirname($configFileName), MAX_PLUGINS_VAR_WRITE_MODE);
            // Copy file
            $ret = @copy($defaultConfigFileName, $configFileName);
            return $ret;
        }
        return false;
    }

    /**
     * A private method to get the host name via by which the Max
     * installation is currently being accessed.
     *
     * @access private
     * @return Mixed A string containing the host name, if available,
     *               false otherwise.
     */
    function _getHostName()
    {
        if (is_null($_SERVER['HTTP_HOST'])) {
            if (is_null($GLOBALS['argv'][1])) {
                MAX::raiseError(MAX_PRODUCT_NAME . " was called via the command line, but had no host as a parameter.\n", PEAR_ERROR_DIE);
                return false;
            }
            $host = $GLOBALS['argv'][1];
        } else {
            if (!empty($_SERVER['HTTP_HOST'])) {
                $host = $_SERVER['HTTP_HOST'];
            } else {
                $host = $_SERVER['SERVER_NAME'];
            }
        }
        return $host;
    }

    /**
     * A private method for recursively creating a direcotry. Used when
     * needed to create directories for cache data and for config files.
     *
     * @access private
     * @param string $directory The directory to create.
     * @param int $mode    Mode (access permissions)
     * @return boolean True on success (directory created, or already
     *                 exists), false otherwise.
     */
    function _mkDirRecursive($directory, $mode = null)
    {
        if (is_dir($directory)) {
            return true;
        } else {
            if (is_null($mode)) {
                $mode = MAX_PLUGINS_VAR_WRITE_MODE;
            }
            $ret1 = MAX_Plugin::_mkDirRecursive(dirname($directory));
            $ret2 = @mkdir($directory, $mode);
            return $ret1 && $ret2;
        }
    }












    /**
     * Factory main module plugin. This method read the module configuration file
     * and (if module config exists and if config contain information about main plugin)
     * return main plugin for module (example: look at geotargeting)
     *
     * @static
     * @param string $module     Module name (plugins/module folder name)
     * @param string $configKey  Name of configuration type which should
     *                           contain name of main plugin
     * @param string $omit       If configKey contain this variable it means
     *                           that main plugin is not defined
     *
     * @return object            Plugin object or false if any error occured
     *
     */
    function &factoryPluginByModuleConfig($module, $configKey = 'type', $omit = 'none')
    {
        // read module configuration file
        $conf = MAX_Plugin::getConfig($module);
        // get plugin type from configuration
        if (!isset($conf[$configKey])) {
            return false;
        } else {
            $packageName = explode(':', $conf[$configKey]);
            if (count($packageName) > 1) {
                $package = $packageName[0];
                $name = $packageName[1];
            } else {
                $package = $conf[$configKey];
                $name = null;
            }
        }
        if ($package == $omit) {
            // prevent from factory non existings class
            return null;
        }
        if (!empty($module) && !empty($package)) {
            return MAX_Plugin::factory($module, $package, $name);
        } else {
            return false;
        }
    }

    /**
     * Write config for plugin. Create also config "fake" files.
     *
     * @param array $aConfig   Array to save as a config
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     *
     * @return bool  True on success else false
     *
     */
    function writePluginConfig($aConfig, $module, $package = null, $name = null)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $configFileName = MAX_Plugin::getConfigFileName($module, $package, $name);
        if (!file_exists($configFileName)) {
            MAX_Plugin::copyDefaultConfig($module, $package, $name);
        }
        include_once 'Config.php';
        $c = new Config();
        $c->parseConfig($aConfig, 'phpArray');
        if (!$c->writeConfig($configFileName, 'inifile')) {
            return false;
        }
        // Create fake files
        $configHosts = array(
            $conf['webpath']['admin'],
            $conf['webpath']['delivery'],
            $conf['webpath']['deliverySSL']
        );
        $hostName = MAX_Plugin::getHostName();
        foreach($configHosts as $host) {
            $url = @parse_url('http://' . $host);
            if (!isset($url['host'])) {
                return false;
            }
            $parsedHost = $url['host'];
            if ($parsedHost == $hostName) {
                continue;
            }
            // Create fake file for this host
            $configFileName = MAX_Plugin::getConfigFileName($module, $package, $name, false, $parsedHost);
            $config = array('realConfig' => $hostName);
            $c = new Config();
            $c->parseConfig($config, 'phpArray');
            if (!$c->writeConfig($configFileName, 'inifile')) {
                return false;
            }
        }
        return true;
    }

    /**
     * Test if a cache is available and (if yes) return it
     *
     * @static
     * @param string  $id       Cache id
     * @param string  $module   Module name (plugins/module folder name)
     * @param string  $package  Package name (plugins/module/package folder namer)
     * @param string  $name     Name of the php file which contains the plugin
     * @param bool    $doNotTestCacheValidity   If set to true, the cache validity won't be tested
     * @param array   $options  Options - see Cache_Lite()
     *
     * @return mixed           Data of the cache (or false if no cache available)
     *
     */
    function getCacheForPluginById($id, $module, $package, $name = null, $doNotTestCacheValidity = true, $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = MAX_Plugin::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        $data = $cache->get($id, $name, $doNotTestCacheValidity);
        return $data;
    }

    /**
     * Save data in plugin cache file
     *
     * @static
     * @param string $data     Data to put in cache (can be another type than strings if automaticSerialization is on)
     * @param string $id       Cache id
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     * @param array  $options  Options - see Cache_Lite()
     *
     * @return bool            True if no problem
     *
     */
    function saveCacheForPlugin($data, $id, $module, $package, $name = null, $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = MAX_Plugin::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        return $cache->save($data, $id, $name);
    }

    /**
     * Delete plugin cache
     *
     * @static
     * @param string $module   Module name (plugins/module folder name)
     * @param string $package  Package name (plugins/module/package folder namer)
     * @param string $name     Name of the php file which contains the plugin
     * @param string $mode     Flush cache mode : 'old', 'ingroup', 'notingroup'
     * @param array  $options  Options - see Cache_Lite()
     *
     * @return bool            True if no problem
     *
     */
    function cleanPluginCache($module, $package, $name = null, $mode = 'ingroup', $options = null)
    {
        include_once 'Cache/Lite.php';
        if ($name === null) {
            $name = $package;
        }
        if ($options === null) {
            $options = MAX_Plugin::prepareCacheOptions($module, $package);
        }
        $cache = new Cache_Lite($options);
        return $cache->clean($name, $mode);
    }

    /**
     * Build PEAR Cache_Lite options for plugin from specified module/package.
     * If cacheDir doesn't exists this method also create it.
     *
     * @param string $module    Module name
     * @param string $package   Package name
     *
     * @see getCacheForPluginById()
     * @see saveCacheForPlugin()
     *
     * @return array    Array with cache options for PEAR Cache_Lite class
     */
    function prepareCacheOptions($module, $package, $cacheDir = null, $cacheExpire = null)
    {
        $options = array(
            'cacheDir' => (is_null($cacheDir)) ? MAX_PLUGINS_CACHE . '/' . $module . '/' . $package . '/cache/' : $cacheDir,
            'lifeTime' => (is_null($cacheExpire)) ? -1 : $cacheExpire,
            'automaticSerialization' => true,
        );
        if (!file_exists($options['cacheDir'])) {
            if (is_writable(MAX_PLUGINS_CACHE)) {
                MAX_Plugin::_mkDirRecursive($options['cacheDir'], MAX_PLUGINS_VAR_WRITE_MODE);
            } else {
                Max::raiseError('Folder: "' . MAX_PLUGINS_CACHE . '" is not writeable.');
            }
        }
        return $options;
    }
}

?>
